<?php
/**
 * Forty_Two_Other_Fonts class
 *
 * @package Forty_Two
 * @license GPL-3.0-or-later
 * @link    https://aspengrovestudios.com/themes/forty-two/
 */

/**
 * Class representing the list of other fonts to enqueue in the theme.
 *
 * @since 1.0.0
 */
final class Forty_Two_Other_Fonts {

	/**
	 * The theme's Forty_Two_Fonts instance.
	 *
	 * @since 1.0.0
	 * @var Forty_Two_Fonts
	 */
	private $themeFonts;

	/**
	 * A Forty_Two_Font instance used to represent the contents of the other font setting's array.
	 *
	 * @since 1.0.0
	 * @var Forty_Two_Font
	 */
	private $otherFont;

	/**
	 * The other fonts setting.
	 *
	 * @since 1.0.0
	 * @var Forty_Two_Array_Setting
	 */
	private $setting;

	/**
	 * Constructor.
	 *
	 * Sets the font definition.
	 *
	 * @since 1.0.0
	 *
	 * @param string $id   Unique string identifier for this font.
	 * @param array  $args Font definition as $prop => $value pairs.
	 */
	public function __construct($themeFonts) {
		$this->themeFonts = $themeFonts;
	}

	/**
	 * Registers the other fonts setting.
	 *
	 * @since 1.0.0
	 *
	 */
	final public function register() {
		$this->otherFont = new Forty_Two_Font('other_fonts', array(
			Forty_Two_Font::PROP_HAS_SIZE => false,
			Forty_Two_Font::PROP_MULTI_WEIGHTS => true
		));
		$this->setting = new Forty_Two_Array_Setting('other_fonts', array(), $this->otherFont->get_setting());
		$settings = $this->themeFonts->get_dependency( 'settings' );
		$settings->register_setting( $this->setting );
		
		add_action('wp_ajax_forty_two_add_other_font', array($this, 'add_font_ajax'));
	}

	/**
	 * Gets the current value for the other fonts setting.
	 *
	 * @since 1.0.0
	 *
	 * @return mixed Current value for the other fonts setting.
	 */
	final public function get_value($flat=true) {
		$settingValue = $this->setting->get_value();
		if (!$flat) {
			return $settingValue;
		}
		
		$fonts = array();
		
		foreach ($settingValue as $font) {
			foreach ($font['weight'] as $weight) {
				$fonts[] = array(
					'family' => $font['family'],
					'weight' => $weight
				);
			}
		}
		
		return $fonts;
	}
	
	/**
	 * Adds a font to the other fonts setting.
	 *
	 * @since 1.0.0
	 *
	 * @return
	 */
	final public function add_font($family, $weight) {
		$settingValue = $this->setting->get_value();
		$fonts = array();
		
		foreach ($settingValue as &$font) {
			if ($font['family'] == $family) {
				if (!in_array($weight, $font['weight'])) {
					$font['weight'][] = $weight;
					$this->setting->set_value($settingValue);
				}
				return true;
			}
		}
		$settingValue[] = array(
			'family' => $family,
			'weight' => array($weight)
		);
		$this->setting->set_value($settingValue);
		return true;
	}
	
	/**
	 * Adds a font to the other fonts setting, via ajax.
	 *
	 * @since 1.0.0
	 *
	 * @return
	 */
	final public function add_font_ajax() {
		if (empty($_POST['family']) || empty($_POST['weight'])) {
			wp_send_json_error();
		}
		$this->add_font($_POST['family'], $_POST['weight']);
		$fontsData = $this->themeFonts->get_font_controls_data();
		wp_send_json_success(array(
			'otherFonts' => $fontsData['otherFonts'],
			'cssUrls' => $this->themeFonts->load_webfonts(true)
		));
	}
}
