<?php
/**
 * Forty_Two_Sidebar class
 *
 * @package Forty_Two
 * @license GPL-3.0-or-later
 * @link    https://aspengrovestudios.com/themes/forty-two/
 */

/**
 * Class responsible for the theme sidebar.
 *
 * @since 1.0.0
 */
class Forty_Two_Sidebar extends Forty_Two_Theme_Component_Base {

	/**
	 * Constructor.
	 *
	 * Sets the required dependencies.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		$this->require_dependency_class( 'Forty_Two_Settings' );
		$this->require_dependency_class( 'Forty_Two_Customizer' );
		$this->require_dependency_class( 'Forty_Two_Widgets' );
		$this->require_dependency_class( 'Forty_Two_Distraction_Free_Mode' );
		$this->require_dependency_class( 'Forty_Two_Content_Types' );
	}

	/**
	 * Gets the name of the sidebar to display on the current page.
	 *
	 * @since 1.0.0
	 *
	 * @return string The sidebar name.
	 */
	public function get_sidebar_name() {
		$sidebar_name = 'primary';

		if ( $this->get_dependency( 'settings' )->get( 'blog_sidebar_enabled' ) && $this->allow_display_blog_sidebar() ) {
			$sidebar_name = 'blog';
		}

		/**
		 * Filters the name of the sidebar to display on the current page.
		 *
		 * @since 1.0.0
		 *
		 * @param string $sidebar_name The sidebar name. By default either 'primary' or 'blog'.
		 */
		return apply_filters( 'forty_two_get_sidebar_name', $sidebar_name );
	}

	/**
	 * Checks whether the sidebar should be displayed.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the sidebar should be displayed, false otherwise.
	 */
	public function should_display_sidebar() {
		if ( ! $this->allow_display_sidebar() ) {
			return false;
		}

		return 'no_sidebar' !== $this->get_dependency( 'settings' )->get( 'sidebar_mode' );
	}

	/**
	 * Checks whether the current page does allow the sidebar to be displayed.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the sidebar can be displayed, false otherwise.
	 */
	public function allow_display_sidebar() {
		$result = ! $this->get_dependency( 'distraction_free_mode' )->is_distraction_free();

		/**
		 * Filters whether to allow displaying the sidebar on the current page.
		 *
		 * @since 1.0.0
		 *
		 * @param bool $result Whether to allow displaying the sidebar on the current page.
		 */
		return apply_filters( 'forty_two_allow_display_sidebar', $result );
	}

	/**
	 * Checks whether the current page does allow the blog sidebar to be displayed.
	 *
	 * @since 1.0.0
	 *
	 * @return bool True if the blog sidebar can be displayed, false otherwise.
	 */
	public function allow_display_blog_sidebar() {
		$post_type = $this->get_dependency( 'content_types' )->detect_post_type();
		$result    = 'post' === $post_type;

		/**
		 * Filters whether to allow displaying the blog sidebar on the current page.
		 *
		 * @since 1.0.0
		 *
		 * @param bool $result Whether to allow displaying the blog sidebar on the current page.
		 */
		return apply_filters( 'forty_two_allow_display_blog_sidebar', $result );
	}

	/**
	 * Gets the available choices for the 'sidebar_mode' setting.
	 *
	 * @since 1.0.0
	 *
	 * @return array Array where values are the keys, and labels are the values.
	 */
	public function get_sidebar_mode_choices() {
		return array(
			'no_sidebar'    => __( 'No Sidebar', 'forty-two' ),
			'left_sidebar'  => __( 'Left Sidebar', 'forty-two' ),
			'right_sidebar' => __( 'Right Sidebar', 'forty-two' ),
		);
	}

	/**
	 * Gets the available choices for the 'sidebar_size' setting.
	 *
	 * @since 1.0.0
	 *
	 * @return array Array where values are the keys, and labels are the values.
	 */
	public function get_sidebar_size_choices() {
		return array(
			'small'  => __( 'Small', 'forty-two' ),
			'medium' => __( 'Medium', 'forty-two' ),
			'large'  => __( 'Large', 'forty-two' ),
		);
	}

	/**
	 * Magic call method.
	 *
	 * Handles the widgets registration action hook callbacks.
	 *
	 * @since 1.0.0
	 *
	 * @param string $method Method name.
	 * @param array  $args   Method arguments.
	 *
	 * @throws BadMethodCallException Thrown when method name is invalid.
	 */
	public function __call( $method, $args ) {
		switch ( $method ) {
			case 'register_settings':
			case 'register_widget_areas':
			case 'register_customize_partial':
			case 'register_customize_controls_js':
			case 'register_customize_preview_js':
				return call_user_func_array( array( $this, $method ), $args );
			case 'add_sidebar_body_classes':
				if ( empty( $args ) ) {
					return;
				}

				$settings = $this->get_dependency( 'settings' );

				$classes = $args[0];
				if ( ! $this->should_display_sidebar() ) {
					$classes[] = 'no-sidebar';
				} else {
					$classes[] = str_replace( '_', '-', $settings->get( 'sidebar_mode' ) );
				}
				$classes[] = 'sidebar-' . $settings->get( 'sidebar_size' );

				return $classes;
			default:
				/* translators: %s: method name */
				throw new BadMethodCallException( sprintf( __( 'Call to undefined method %s', 'forty-two' ), __CLASS__ . '::' . $method . '()' ) );
		}
	}

	/**
	 * Registers settings for sidebar behavior.
	 *
	 * @since 1.0.0
	 */
	protected function register_settings() {
		$settings = $this->get_dependency( 'settings' );

		$settings->register_setting( new Forty_Two_Enum_String_Setting(
			'sidebar_mode',
			array(
				Forty_Two_Enum_String_Setting::PROP_ENUM    => array_keys( $this->get_sidebar_mode_choices() ),
				Forty_Two_Enum_String_Setting::PROP_DEFAULT => 'right_sidebar',
			)
		) );

		$settings->register_setting( new Forty_Two_Enum_String_Setting(
			'sidebar_size',
			array(
				Forty_Two_Enum_String_Setting::PROP_ENUM    => array_keys( $this->get_sidebar_size_choices() ),
				Forty_Two_Enum_String_Setting::PROP_DEFAULT => 'medium',
			)
		) );

		$settings->register_setting( new Forty_Two_Boolean_Setting(
			'blog_sidebar_enabled',
			array( Forty_Two_Boolean_Setting::PROP_DEFAULT => false )
		) );
	}

	/**
	 * Registers widget areas for the sidebar.
	 *
	 * @since 1.0.0
	 * @param Forty_Two_Widgets $widgets Widgets handler instance.
	 */
	protected function register_widget_areas( $widgets ) {
		$widgets->register_widget_area( new Forty_Two_Widget_Area(
			'primary',
			array(
				Forty_Two_Widget_Area::PROP_TITLE       => __( 'Primary Sidebar', 'forty-two' ),
				Forty_Two_Widget_Area::PROP_DESCRIPTION => __( 'Add widgets here to appear in the sidebar for your main content.', 'forty-two' ),
			)
		) );

		$blog_sidebar_description = __( 'Add widgets here to appear in the sidebar for blog posts and archive pages.', 'forty-two' );
		if ( ! $this->get_dependency( 'settings' )->get( 'blog_sidebar_enabled' ) ) {
			if ( 'customize.php' === $GLOBALS['pagenow'] ) {
				$blog_sidebar_description .= ' <a href="" onclick="wp.customize.control( \'blog_sidebar_enabled\' ).focus();return false;">' . __( 'You need to enable the sidebar first.', 'forty-two' ) . '</a>';
			} else {

				// WordPress only allows arbitrary HTML here since version 4.9.7.
				if ( version_compare( $GLOBALS['wp_version'], '4.9.7', '>=' ) ) {
					$blog_sidebar_description .= ' <a href="' . esc_url( admin_url( 'customize.php?autofocus[control]=blog_sidebar_enabled' ) ) . '">' . __( 'You need to enable the sidebar in the Customizer first.', 'forty-two' ) . '</a>';
				} else {
					$blog_sidebar_description .= ' ' . __( 'You need to enable the sidebar in the Customizer first.', 'forty-two' );
				}
			}
		}

		$widgets->register_widget_area( new Forty_Two_Widget_Area(
			'blog',
			array(
				Forty_Two_Widget_Area::PROP_TITLE       => __( 'Blog Sidebar', 'forty-two' ),
				Forty_Two_Widget_Area::PROP_DESCRIPTION => $blog_sidebar_description,
			)
		) );
	}

	/**
	 * Registers Customizer partial.
	 *
	 * @since 1.0.0
	 *
	 * @param Forty_Two_Customizer $customizer Customizer instance.
	 */
	protected function register_customize_partial( $customizer ) {
		/*
		TODO: Implement these active callbacks via JavaScript. Then remove this method.
		$customizer->add_control( 'sidebar_mode', array(
			Forty_Two_Customize_Control::PROP_SECTION         => Forty_Two_Widgets::CUSTOMIZER_SECTION,
			Forty_Two_Customize_Control::PROP_TITLE           => __( 'Sidebar Mode', 'forty-two' ),
			Forty_Two_Customize_Control::PROP_DESCRIPTION     => __( 'Specify if and how the sidebar should be displayed.', 'forty-two' ),
			Forty_Two_Customize_Control::PROP_TYPE            => Forty_Two_Customize_Control::TYPE_RADIO,
			Forty_Two_Customize_Control::PROP_CHOICES         => $this->get_sidebar_mode_choices(),
			Forty_Two_Customize_Control::PROP_ACTIVE_CALLBACK => array( $this, 'allow_display_sidebar' ),
		) );

		$customizer->add_control( 'sidebar_size', array(
			Forty_Two_Customize_Control::PROP_SECTION         => Forty_Two_Widgets::CUSTOMIZER_SECTION,
			Forty_Two_Customize_Control::PROP_TITLE           => __( 'Sidebar Size', 'forty-two' ),
			Forty_Two_Customize_Control::PROP_DESCRIPTION     => __( 'Specify the width of the sidebar.', 'forty-two' ),
			Forty_Two_Customize_Control::PROP_TYPE            => Forty_Two_Customize_Control::TYPE_RADIO,
			Forty_Two_Customize_Control::PROP_CHOICES         => $this->get_sidebar_size_choices(),
			Forty_Two_Customize_Control::PROP_ACTIVE_CALLBACK => array( $this, 'allow_display_sidebar' ),
		) );

		$customizer->add_control( 'blog_sidebar_enabled', array(
			Forty_Two_Customize_Control::PROP_SECTION         => Forty_Two_Widgets::CUSTOMIZER_SECTION,
			Forty_Two_Customize_Control::PROP_TITLE           => __( 'Enable Blog Sidebar?', 'forty-two' ),
			Forty_Two_Customize_Control::PROP_DESCRIPTION     => __( 'If you enable the blog sidebar, it will be shown beside your blog and single post content instead of the primary sidebar.', 'forty-two' ),
			Forty_Two_Customize_Control::PROP_TYPE            => Forty_Two_Customize_Control::TYPE_CHECKBOX,
			Forty_Two_Customize_Control::PROP_ACTIVE_CALLBACK => array( $this, 'allow_display_blog_sidebar' ),
		) );
		*/

		$customizer->add_partial( 'blog_sidebar_enabled', array(
			Forty_Two_Customize_Partial::PROP_SELECTOR            => '#sidebar',
			Forty_Two_Customize_Partial::PROP_RENDER_CALLBACK     => 'get_sidebar',
			Forty_Two_Customize_Partial::PROP_CONTAINER_INCLUSIVE => true,
		) );
	}

	/**
	 * Registers scripts for the Customizer controls.
	 *
	 * @since 1.0.0
	 *
	 * @param Forty_Two_Assets $assets Assets instance.
	 */
	protected function register_customize_controls_js( $assets ) {
		/**
		 * Filters the data to pass to the sidebar controls script.
		 *
		 * @since 1.0.0
		 *
		 * @param array $sidebar_controls_data Associative data array to pass to the script.
		 */
		$sidebar_controls_data = apply_filters( 'forty_two_sidebar_controls_data', array(
			'sidebarModeChoices' => $this->get_sidebar_mode_choices(),
			'sidebarSizeChoices' => $this->get_sidebar_size_choices(),
		) );

		$assets->register_asset( new Forty_Two_Script(
			'forty-two-sidebar-customize-controls',
			get_theme_file_uri( '/assets/dist/js/sidebar.customize-controls.js' ),
			array(
				Forty_Two_Script::PROP_DEPENDENCIES => array( 'customize-controls', 'wp-i18n' ),
				Forty_Two_Script::PROP_VERSION      => FORTY_TWO_VERSION,
				Forty_Two_Script::PROP_LOCATION     => Forty_Two_Script::LOCATION_CUSTOMIZE_CONTROLS,
				Forty_Two_Script::PROP_MIN_URI      => true,
				Forty_Two_Script::PROP_DATA_NAME    => 'themeSidebarControlsData',
				Forty_Two_Script::PROP_DATA         => $sidebar_controls_data,
			)
		) );
	}

	/**
	 * Registers scripts for the Customizer preview.
	 *
	 * @since 1.0.0
	 *
	 * @param Forty_Two_Assets $assets Assets instance.
	 */
	protected function register_customize_preview_js( $assets ) {
		$assets->register_asset( new Forty_Two_Script(
			'forty-two-sidebar-customize-preview',
			get_theme_file_uri( '/assets/dist/js/sidebar.customize-preview.js' ),
			array(
				Forty_Two_Script::PROP_DEPENDENCIES => array( 'customize-preview' ),
				Forty_Two_Script::PROP_VERSION      => FORTY_TWO_VERSION,
				Forty_Two_Script::PROP_LOCATION     => Forty_Two_Script::LOCATION_CUSTOMIZE_PREVIEW,
				Forty_Two_Script::PROP_MIN_URI      => true,
				Forty_Two_Script::PROP_DATA_NAME    => 'themeSidebarPreviewData',
				Forty_Two_Script::PROP_DATA         => array(
					'sidebarModeChoices' => $this->get_sidebar_mode_choices(),
					'sidebarSizeChoices' => $this->get_sidebar_size_choices(),
				),
			)
		) );
	}

	/**
	 * Adds hooks and runs other processes required to initialize the component.
	 *
	 * @since 1.0.0
	 */
	protected function run_initialization() {
		add_action( 'after_setup_theme', array( $this, 'register_settings' ), 10, 0 );
		add_filter( 'body_class', array( $this, 'add_sidebar_body_classes' ), 10, 1 );

		$widgets = $this->get_dependency( 'widgets' );
		$widgets->on_init( array( $this, 'register_widget_areas' ) );

		$customizer = $this->get_dependency( 'customizer' );
		$customizer->on_init( array( $this, 'register_customize_partial' ) );
		$customizer->on_js_controls_init( array( $this, 'register_customize_controls_js' ) );
		$customizer->on_js_preview_init( array( $this, 'register_customize_preview_js' ) );
	}
}
