<?php
/**
 * WooCommerce compatibility functions
 *
 * @package Forty_Two
 * @license GPL-3.0-or-later
 * @link    https://aspengrovestudios.com/themes/forty-two/
 */

add_filter( 'woocommerce_enqueue_styles', '__return_empty_array' );

/**
 * Registers support for various WooCommerce features.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_setup() {
	add_theme_support( 'woocommerce' );
	add_theme_support( 'wc-product-gallery-zoom' );
	add_theme_support( 'wc-product-gallery-lightbox' );
	add_theme_support( 'wc-product-gallery-slider' );
}
add_action( 'after_setup_theme', 'forty_two_woocommerce_setup', 11 );

/**
 * Registers the shop sidebar.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_register_sidebar() {
	$sidebar_description = __( 'Add widgets here to appear in the sidebar for shop content.', 'forty-two' );
	if ( ! get_theme_mod( 'shop_sidebar_enabled' ) ) {
		// If core allowed simple HTML here, a link to the respective customizer control would surely help.
		$sidebar_description .= ' ' . __( 'You need to enable the sidebar in the Customizer first.', 'forty-two' );
	}

	register_sidebar( array(
		'name'          => __( 'Shop Sidebar', 'forty-two' ),
		'id'            => 'shop',
		'description'   => $sidebar_description,
		'before_widget' => '<section id="%1$s" class="widget %2$s">',
		'after_widget'  => '</section>',
		'before_title'  => '<h2 class="widget-title">',
		'after_title'   => '</h2>',
	) );
}
add_action( 'widgets_init', 'forty_two_woocommerce_register_sidebar', 11 );

/**
 * Adds an inline stylesheet to the main stylesheet to load the WooCommerce star font.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_enqueue_scripts() {
	$font_path   = WC()->plugin_url() . '/assets/fonts/';
	$inline_font = '@font-face {
			font-family: "star";
			src: url("' . $font_path . 'star.eot");
			src: url("' . $font_path . 'star.eot?#iefix") format("embedded-opentype"),
				url("' . $font_path . 'star.woff") format("woff"),
				url("' . $font_path . 'star.ttf") format("truetype"),
				url("' . $font_path . 'star.svg#star") format("svg");
			font-weight: normal;
			font-style: normal;
		}';

	wp_add_inline_style( 'forty-two-style', $inline_font );
}
add_action( 'wp_enqueue_scripts', 'forty_two_woocommerce_enqueue_scripts', 11 );

/**
 * Registers WooCommerce sidebar settings.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_register_settings() {
	$settings = forty_two( 'settings' );

	$settings->register_setting( new Forty_Two_Boolean_Setting(
		'shop_sidebar_enabled',
		array( Forty_Two_Boolean_Setting::PROP_DEFAULT => false )
	) );
}
add_action( 'after_setup_theme', 'forty_two_woocommerce_register_settings', 10, 0 );

/**
 * Sets a flag so that the shop sidebar control is rendered in the Customizer.
 *
 * @since 1.0.0
 *
 * @param array $data Associative data array to pass to the script.
 * @return array Filtered array.
 */
function forty_two_woocommerce_set_sidebar_control_flag( $data ) {
	$data['displayShopSidebarEnabledSetting'] = true;

	return $data;
}
add_filter( 'forty_two_sidebar_controls_data', 'forty_two_woocommerce_set_sidebar_control_flag', 10, 1 );

/**
 * Registers WooCommerce sidebar Customizer partial.
 *
 * @since 1.0.0
 *
 * @param Forty_Two_Customizer $customizer Customizer instance.
 */
function forty_two_woocommerce_customize_register( $customizer ) {
	$customizer->add_partial( 'shop_sidebar_enabled', array(
		Forty_Two_Customize_Partial::PROP_SELECTOR            => '#sidebar',
		Forty_Two_Customize_Partial::PROP_RENDER_CALLBACK     => 'get_sidebar',
		Forty_Two_Customize_Partial::PROP_CONTAINER_INCLUSIVE => true,
	) );
}
forty_two( 'customizer' )->on_init( 'forty_two_woocommerce_customize_register' );

/**
 * Overrides the name of the sidebar to display on the current page, if necessary.
 *
 * @since 1.0.0
 *
 * @param string $sidebar_name Sidebar name.
 * @return string Modified sidebar name.
 */
function forty_two_woocommerce_maybe_override_sidebar_name( $sidebar_name ) {
	if ( get_theme_mod( 'shop_sidebar_enabled' ) && is_woocommerce() ) {
		return 'shop';
	}

	return $sidebar_name;
}
add_filter( 'forty_two_get_sidebar_name', 'forty_two_woocommerce_maybe_override_sidebar_name' );

/**
 * Overrides whether the current page should be displayed in distraction-free mode, if necessary.
 *
 * @since 1.0.0
 *
 * @param bool $distraction_free Condition result.
 * @return bool Modified condition result.
 */
function forty_two_woocommerce_maybe_set_distraction_free( $distraction_free ) {
	if ( is_cart() || is_checkout() ) {
		return true;
	}

	return $distraction_free;
}
add_filter( 'forty_two_is_distraction_free', 'forty_two_woocommerce_maybe_set_distraction_free' );

/**
 * Adds a 'woocommerce-active' class to the body tag.
 *
 * @since 1.0.0
 *
 * @param array $classes Classes for the body element.
 * @return array Modified classes.
 */
function forty_two_woocommerce_active_body_class( $classes ) {
	$classes[] = 'woocommerce-active';

	return $classes;
}
add_filter( 'body_class', 'forty_two_woocommerce_active_body_class' );

/**
 * Filters the products to show per page.
 *
 * @since 1.0.0
 *
 * @return int Number of products.
 */
function forty_two_woocommerce_products_per_page() {
	return 12;
}
add_filter( 'loop_shop_per_page', 'forty_two_woocommerce_products_per_page' );

/**
 * Filters the product gallery thumnbail columns.
 *
 * @since 1.0.0
 *
 * @return int Number of columns.
 */
function forty_two_woocommerce_thumbnail_columns() {
	return 4;
}
add_filter( 'woocommerce_product_thumbnails_columns', 'forty_two_woocommerce_thumbnail_columns' );

/**
 * Filters the default loop columns on product archives.
 *
 * @since 1.0.0
 *
 * @return int Number of products per row.
 */
function forty_two_woocommerce_loop_columns() {
	return 3;
}
add_filter( 'loop_shop_columns', 'forty_two_woocommerce_loop_columns' );

/**
 * Filters the related products arguments.
 *
 * @since 1.0.0
 *
 * @param array $args Related products arguments.
 * @return array Modified related products arguments.
 */
function forty_two_woocommerce_related_products_args( $args ) {
	$defaults = array(
		'posts_per_page' => 3,
		'columns'        => 3,
	);

	$args = wp_parse_args( $defaults, $args );

	return $args;
}
add_filter( 'woocommerce_output_related_products_args', 'forty_two_woocommerce_related_products_args' );

/**
 * Prints the product columns wrapper opening tag.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_product_columns_wrapper() {
	$columns = forty_two_woocommerce_loop_columns();

	echo '<div class="columns-' . absint( $columns ) . '">';
}
add_action( 'woocommerce_before_shop_loop', 'forty_two_woocommerce_product_columns_wrapper', 40 );

/**
 * Prints the product columns wrapper closing tag.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_product_columns_wrapper_close() {
	echo '</div>';
}
add_action( 'woocommerce_after_shop_loop', 'forty_two_woocommerce_product_columns_wrapper_close', 40 );

/**
 * Prints the opening tags of the wrapper for all WooCommerce content.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_wrapper_before() {
	?>
	<main id="main" class="site-main">
	<?php
}
add_action( 'woocommerce_before_main_content', 'forty_two_woocommerce_wrapper_before' );
remove_action( 'woocommerce_before_main_content', 'woocommerce_output_content_wrapper' );

/**
 * Prints the closing tags of the wrapper for all WooCommerce content.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_wrapper_after() {
	?>
	</main><!-- #main -->
	<?php
}
add_action( 'woocommerce_after_main_content', 'forty_two_woocommerce_wrapper_after' );
remove_action( 'woocommerce_after_main_content', 'woocommerce_output_content_wrapper_end' );

// Call the regular sidebar template instead of the WooCommerce one.
add_action( 'woocommerce_sidebar', 'get_sidebar' );
remove_action( 'woocommerce_sidebar', 'woocommerce_get_sidebar' );

/**
 * Renders a cart link.
 *
 * Displays a link to the cart including the number of items present and the cart total.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_cart_link() {
	$subtotal = WC()->cart->get_cart_subtotal();
	$count    = WC()->cart->get_cart_contents_count();

	?>
	<a class="cart-contents" href="<?php echo esc_url( wc_get_cart_url() ); ?>" title="<?php esc_attr_e( 'View your shopping cart', 'forty-two' ); ?>">
		<span class="amount">
			<?php echo wp_kses_data( $subtotal ); ?>
		</span>
		<span class="count">
			<?php
			echo wp_kses_data( sprintf(
				/* translators: %s: number of items in the cart */
				_n( '%s item', '%s items', $count, 'forty-two' ),
				number_format_i18n( $count )
			) );
			?>
		</span>
	</a>
	<?php
}

/**
 * Displays the cart content for the header.
 *
 * @since 1.0.0
 */
function forty_two_woocommerce_header_cart() {
	if ( is_cart() ) {
		$class = 'current-menu-item';
	} else {
		$class = '';
	}
	?>
	<ul id="site-header-cart" class="site-header-cart">
		<li class="<?php echo esc_attr( $class ); ?>">
			<?php forty_two_woocommerce_cart_link(); ?>
		</li>
		<li>
			<?php
				$instance = array(
					'title' => '',
				);

				the_widget( 'WC_Widget_Cart', $instance );
			?>
		</li>
	</ul>
	<?php
}
add_action( 'forty_two_after_site_navigation', 'forty_two_woocommerce_header_cart' );

/**
 * Filters the cart fragments.
 *
 * Ensures cart contents update when products are added to the cart via AJAX.
 *
 * @since 1.0.0
 *
 * @param array $fragments Fragments to refresh via AJAX.
 * @return array Fragments to refresh via AJAX.
 */
function forty_two_woocommerce_cart_link_fragment( $fragments ) {
	ob_start();
	forty_two_woocommerce_cart_link();
	$fragments['a.cart-contents'] = ob_get_clean();

	return $fragments;
}
add_filter( 'woocommerce_add_to_cart_fragments', 'forty_two_woocommerce_cart_link_fragment' );
